﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2009.06.14.
 * Idő: 9:11
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;
using System.IO;
using System.Drawing;
//
using TetheredSun.Controls;
using TetheredSun.Mathematics;

namespace TetheredSun.GnuPlot
{
	/// <summary>
	/// Description of GnuPlotTextWriter.
	/// </summary>
	public class GnuPlotTextWriter : StreamWriter
	{
		protected Axis axisX = new Axis("x");
		protected Axis axisY = new Axis("y");
		protected Axis axisZ = new Axis("z");
		protected FontInfo baseFont = new FontInfo("Helvetica", 12.0f);
		protected string outputPath = "Plot.eps";
		protected Terminal terminal;
		
		
		public GnuPlotTextWriter(Stream stream, Terminal terminal) : base(stream)
		{ 
			this.terminal = terminal;
		}
		
		public GnuPlotTextWriter(string path, Terminal terminal) : base(path)
		{
			this.terminal = terminal;
		}
		
		public GnuPlotTextWriter(string path) : this(path, Terminal.EpsMonochrome) { }
		
		
		public Axis AxisX {
			get { return axisX; }
			set { axisX = value; }
		}
		
		public Axis AxisY {
			get { return axisY; }
			set { axisY = value; }
		}
		
		public Axis AxisZ {
			get { return axisZ; }
			set { axisZ = value; }
		}
		
		public FontInfo BaseFont {
			get { return baseFont; }
			set { baseFont = value; }
		}
		
		public string OutputPath {
			get { return outputPath; }
			set { outputPath = value; }
		}
		
		public Terminal Terminal {
			get { return terminal; }
			set { terminal = value; }
		}
		
		
		public static string Quote(string text)
		{
			return String.IsNullOrEmpty(text) ? text : "\"" + text + "\"";
		}
		
		public void WriteAxisX()
		{
			if (axisX != null) this.Write(axisX.ToGnuPlotString(AxisType.X));
		}
		
		public void WriteAxisY()
		{
			if (axisY != null) this.Write(axisY.ToGnuPlotString(AxisType.Y));
		}
		
		public void WriteAxisZ()
		{
			if (axisZ != null) this.Write(axisZ.ToGnuPlotString(AxisType.Z));
		}
		
		public void WriteBorder(Border border)
		{
			this.WriteLine("set border " + ((int)border).ToString());
		}
		
		public void WriteComment(string comment)
		{
			this.WriteLine("# " + comment);
		}
		
		public void WriteCurveList(CurveList curveList)
		{
			if (curveList == null) return;
			
			bool first = true;
			bool errorBars = false;
			DataPoint point;
			
			for (int i = 0; i < curveList.Count; i++) {
				if (!curveList[i].Visible) continue;
				if (curveList[i] == null) continue;
				if (curveList[i].Points == null) continue;
				if (curveList[i].Points.Count < 1) continue;
				WritePlot(curveList[i], first);
				first = false;
			}
			this.WriteLine();
			for (int i = 0; i < curveList.Count; i++) {
				if (!curveList[i].Visible) continue;
				if (curveList[i] == null) continue;
				if (curveList[i].Points == null) continue;
				if (curveList[i].Points.Count < 1) continue;
				errorBars = (curveList[i].ErrorBar != null);
				for (int j = 0; j < curveList[i].Points.Count; j++) {
					if (errorBars) {
						point = curveList[i].Points[j] as DataPoint;
						this.WriteLine("{0}\t{1}\t{2}", curveList[i].Points[j].X.ToString("E4").Replace(',', '.'),
						               curveList[i].Points[j].Y.ToString("E4").Replace(',', '.'),
						               (point != null) ? point.Error.ToString("E4").Replace(',', '.') : "0.0");
					} else {
						this.WriteLine("{0}\t{1}", curveList[i].Points[j].X.ToString("E4").Replace(',', '.'),
						               curveList[i].Points[j].Y.ToString("E4").Replace(',', '.'));
					}
				}
				this.WriteLine("end");
			}
		}
		
		public void WriteInlineData(double[][] data, string format, string separator)
		{
			if (data == null) return;
			string line;
			
			for (int i = 0; i < data.Length; i++) {
				if ((data[i] == null) || (data[i].Length <= 0)) continue;
				line = data[i][0].ToString(format).Replace(',', '.');
				for (int j = 1; j < data[i].Length; j++) {
					line += separator + data[i][j].ToString(format).Replace(',', '.');
				}
				this.WriteLine(line);
			}
			this.WriteLine();
			this.WriteLine("e");
		}
		
		public void WriteInlineData(double[][] data)
		{
			WriteInlineData(data, "f3", "\t");
		}
		
		public void WriteInlineData(Complex[][] data, string format, string separator)
		{
			if (data == null) return;
			double[][] absoluteValue = new double[data.Length][];
			
			for (int i = 0; i < absoluteValue.Length; i++) {
				absoluteValue[i] = new double[data[i].Length];
				for (int j = 0; j < data[i].Length; j++) {
					absoluteValue[i][j] = data[i][j].Magnitude;
				}
			}
			
			WriteInlineData(absoluteValue, format, separator);
		}
		
		public void WriteInlineData(Complex[][] data)
		{
			WriteInlineData(data, "f3", "\t");
		}
		
		public void WriteXYData(double[] x, double[][] y, string format, string separator)
		{
			if (x == null) return;
			if (y == null) return;
			string line;
			
			for (int i = 0; i < x.Length; i++) {
				line = x[i].ToString(format).Replace(',', '.');
				for (int j = 0; j < y.Length; j++) {
					line += separator + y[j][i].ToString(format).Replace(',', '.');
				}
				this.WriteLine(line);
			}
			this.WriteLine("e");
		}
		
		public void WriteXYData(double[] x, double[][] y)
		{
			WriteXYData(x, y, "f3", "\t");
		}
		
		public void WriteOutput(bool includePath)
		{
			this.Write("set output");
			if (includePath && !String.IsNullOrEmpty(outputPath)) {
				this.WriteLine(" " + Quote(outputPath));
			} else {
				this.WriteLine();
			}
		}
		
		public void WritePlot(PlotInfo plot, bool first)
		{
			this.Write(first ? String.Format("plot {0}", plot) : String.Format(", {0}", plot));
		}
		
		public void WritePlot(Curve curve, bool first)
		{
			WritePlot(new PlotInfo(curve), first);
		}
		
		public void WriteSeparatorLine(int length)
		{
			string text = String.Empty;
			
			for (int i = 0; i < length; i++) {
				text += "#";
			}
			
			this.WriteLine(text);
		}
				
		public void WriteTerminal()
		{
			string text = "set terminal";
			
			switch (terminal) {
				case Terminal.EpsColour:
					text += " postscript eps enhanced colour";
					break;
				case Terminal.EpsMonochrome:
				default:
					text += " postscript eps enhanced monochrome";
					break;
			}
			text += String.Format(" {0} {1}", Quote(baseFont.Name), baseFont.Size);
			this.WriteLine(text);
		}
	
		public void WriteVariable(string name, string value)
		{
			if (String.IsNullOrEmpty(name)) return;
			
			this.WriteLine(String.Format("{0} = {1}", name, value));
		}
		
		public void WriteVariable(string name, double value)
		{
			WriteVariable(name, value.ToString().Replace(',', '.'));
		}
	
		public static string GetColourString(Color colour)
		{
			string colourString = "#";
			
			colourString += colour.R.ToString("X2");
			colourString += colour.G.ToString("X2");
			colourString += colour.B.ToString("X2");
			
			return colourString;
		}
	}
}
