﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.09.17.
 * Time: 12:16
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
//
using TetheredSun.Mathematics;

namespace TetheredSun.SignalAnalysis
{
	/// <summary>
	/// Description of UnevenSignal.
	/// </summary>
	public class UnevenSignal : ISignal
	{
		public List<double> x;
		public List<double> y;
		
		public UnevenSignal(IList<double> x, IList<double> y)
		{
			this.x = (x == null) ? new List<double>() : new List<double>(x);
			this.y = (y == null) ? new List<double>() : new List<double>(y);
		}
		
		public UnevenSignal(int capacity)
		{
			x = new List<double>(capacity);
			y = new List<double>(capacity);
		}
		
		
		public IList<double> X {
			get { return x; }
		}
		
		public IList<double> Y {
			get { return y; }
		}
		
		public double Mean {
			get { return y.GetMean(); }
		}
		
		public double this[int index] {
			get { return y[index]; }
			set { y[index] = value; }
		}
		
		public int Count {
			get { return y.Count; }
		}
		
		public bool IsReadOnly {
			get { return (y as IList<double>).IsReadOnly; }
		}
		

		public ISignal Reciprocal()
		{
			UnevenSignal result = new UnevenSignal(x.Count);
			
			for (int i = 0; i < x.Count; i++) {
				result.x[i] = x[i];
				result.y[i] = 1.0 / y[i];
			}
			
			return result;
		}
		
		public ISignal SubtractMean()
		{
			UnevenSignal result = new UnevenSignal(x.Count);
			double mean = this.GetMean();
			
			for (int i = 0; i < x.Count; i++) {
				result.x[i] = x[i];
				result.y[i] = y[i] - mean;
			}
			
			return result;
		}
		
		public int IndexOf(double item)
		{
			return y.IndexOf(item);
		}
		
		public void Insert(int index, double item)
		{
			throw new InvalidOperationException("The use of this method is not allowed in this class.");
		}
		
		public void Insert(int index, double xItem, double yItem)
		{
			x.Insert(index, xItem);
			y.Insert(index, yItem);
		}
		
		public void RemoveAt(int index)
		{
			x.RemoveAt(index);
			y.RemoveAt(index);
		}
		
		public void Add(double item)
		{
			throw new InvalidOperationException("The use of this method is not allowed in this class.");
		}
		
		public void Add(double xItem, double yItem)
		{
			x.Add(xItem);
			y.Add(yItem);
		}
		
		public void Clear()
		{
			x.Clear();
			y.Clear();
		}
		
		public bool Contains(double item)
		{
			return y.Contains(item);
		}
		
		public void CopyTo(double[] array, int arrayIndex)
		{
			y.CopyTo(array, arrayIndex);
		}
		
		public bool Remove(double item)
		{
			int index = y.IndexOf(item);
			
			if (index < 0) return false;
			
			if (y.Remove(item)) {
				x.RemoveAt(index);
				return true;
			} else {
				return false;
			}
		}
		
		public IEnumerator<double> GetEnumerator()
		{
			return y.GetEnumerator();
		}
		
		IEnumerator IEnumerable.GetEnumerator()
		{
			return (y as IEnumerable).GetEnumerator();
		}
		
	}
}
